#include <mrt.h>
#include "propertyList.h"
#include "salamanderInterface.h"

#define DEFAULT_SALY_SERVER "salamander.merit.edu"
#define DEFAULT_SALY_PORT   9000

// getopt stuff
extern char	*optarg;
extern int	optind,
		opterr,
		optopt;

char		USAGE_STRING[] =
			"\n%s is used to test the salamander server\n"
			"\nUsage: %s [-s <server>] [-p <port>]"
			"<-k> <channel>\n\n"
			"-s specify salamander server\n"
			"\t(DEFAULT %s)\n"
			"-p specify port number\n"
			"\t(DEFAULT %u)\n"
			"-k persistent\n\n";


int
main (int argc, char *argv[]) {

	salamander_t	*sal;

	trace_t		*TracePtr;
	char			*SalyServerStr;
	char			*ChannelStr;
	u_short			SalyPort;
	prefix_t		*server_prefix;
	plist_t		plist;
	void			*DataPtr;
	u_long		DataLen;
	int			ret;
	int			KeepOpen = 0;

	char			c;

	// Init defaults
	TracePtr = New_Trace();
	set_trace(TracePtr, TRACE_FLAGS, TR_ERROR | NORM,
		TRACE_LOGFILE, "stdout", NULL);

	SalyServerStr = DEFAULT_SALY_SERVER;
	SalyPort      = DEFAULT_SALY_PORT;
	ChannelStr = NULL;

	init_mrt(TracePtr);

	if (argc < 2) {
		trace(NORM, TracePtr, USAGE_STRING,
			argv[0], argv[0], DEFAULT_SALY_SERVER,
			DEFAULT_SALY_PORT);
		return 0xbadbad;
	}

	while ((c = getopt(argc, argv, "s:p:k")) != -1) {
		switch (c) {
		case 's':
			SalyServerStr = optarg;
			break;
		case 'p':
			SalyPort = atoi(optarg);
			break;
		case 'k':
			KeepOpen = 1;
			break;
		}
	}

	for ( ; optind < argc; optind++)
		ChannelStr = argv[optind];

	if (!ChannelStr) {
		trace(NORM, TracePtr, USAGE_STRING,
			argv[0], argv[0], DEFAULT_SALY_SERVER,
			DEFAULT_SALY_PORT);
		return 0xbadbad;
	}

	server_prefix = string_toprefix(SalyServerStr, NULL);
	if (!server_prefix) {
		trace(NORM, TracePtr, "Error finding server\n");
		return 0xdeadbeef;
	}

	// Open connection
	sal = New_Salamander(SalyPort);
	ret = connectToSalamanderServer(sal, server_prefix);
	if (ret != SALAMANDER_OK) {
		trace(NORM, TracePtr, "Error connecting to server\n");
		return 0xbadbad;
	}

	// Send request
	DataPtr = NULL;
	DataLen = 0;
	plist   = createPropertyList();
	updateProperty(plist, "COMMAND", "SUBSCRIBE");
	updateProperty(plist, "NAME", ChannelStr);
	updateProperty(plist, "ID", "1");

	ret = salamanderSendServerData(sal, plist, DataPtr, DataLen);
	if (ret != SALAMANDER_OK) {
		trace(NORM, TracePtr, "Error sending data\n");
		return 0xabbadaba;
	}

	dumpPropertyList(plist);
	destroyPropertyList(plist);

	// Read data
	do {
		plist   = createPropertyList();
		ret = salamanderReadServerData(sal, &plist, &DataPtr, &DataLen);
		if (ret != SALAMANDER_OK) {
			trace(NORM, TracePtr, "Error recving data\n");
			break;
		}

		dumpPropertyList(plist);
		destroyPropertyList(plist);
		if (DataLen) {
			fprintf(stdout, "%s\n", (char *)DataPtr);
			free(DataPtr);
		}
	} while (KeepOpen);

	disconnectFromSalamanderServer(sal);
	Delete_Prefix(server_prefix);

	return 0;
}
