/*
 * property list header file.
 *
 * G. Robert Malan (rmalan@eecs.umich.edu)
 * 2/20/97
 *
 * $Id: propertyList.h,v 1.1.1.1 2000/08/14 18:46:13 labovit Exp $
 */

#ifndef _PROPERTY_H_
#define _PROPERTY_H_ 1

/***********************************************************************
 * Some Well-Known Properties:
 ***********************************************************************/

#define NAME_PROPERTY		"NAME"
#define TIMESTAMP_PROPERTY	"TIMESTAMP"
#define COOKIE_PROPERTY		"COOKIE"

/***********************************************************************
 * System properties:
 ***********************************************************************/

#define OBJ_SIZE_PROPERTY	"OBJ_SIZE"
#define COMMAND_PROPERTY	"COMMAND"
#define ID_PROPERTY			"ID"
#define TIME_PROPERTY		"TIME"

/***********************************************************************
 * Supplier and Client commands (COMMAND_PROPERTY value's):
 ***********************************************************************/

/*
 * Client to server messages:
 */
#define PUBLISH_COMMAND	"PUBLISH"

#define QUERY_COMMAND	"QUERY"
#define REMOVE_QUERY	"REMOVE QUERY"

/*
 * Server to client messages:
 */
#define QUERY_RESPONSE	"QUERY RESPONSE"

#define NOTIFICATION	"NOTIFICATION"

/***********************************************************************
 * Query/Supply Properties and values:
 ***********************************************************************/
#define QUERY_ID		"QUERY ID"
#define QUERY_FAILED	"QUERY FAILED"

/***********************************************************************
 * Data struct for current implementation.
 ***********************************************************************/

typedef struct plist_entry {
	char *key;
	char *value;
	struct plist_entry *previous;
	struct plist_entry *next;
} *plist_entry_t;

typedef struct plist {
	plist_entry_t head;
} *plist_t;

/***********************************************************************
 * Prototypes:
 ***********************************************************************/

#ifdef  __cplusplus
extern "C" {
#endif

/* Interface 1.0+ routines: */
plist_t createPropertyList(void);
void destroyPropertyList(plist_t plist);
int updateProperty(plist_t plist, char *key, char *value);
plist_t copyPropertyList(plist_t from);
void deleteProperty(plist_t list, char *key);
char *getProperty(plist_t plist, char *key);
int iteratePropertyList(plist_t plist, plist_entry_t * cookie,
		char **key, char **value);
int plistEqual(plist_t one, plist_t two);
void dumpPropertyList(plist_t plist);

/* Interface 2.0 routines: */
char *marshallPlist(plist_t plist, unsigned long *length);
plist_t unmarshallPlist(char *mashalledPlist, unsigned long plist_length);

#ifdef  __cplusplus
}
#endif

#endif /* _PROPERTY_H_ */
